@@ -35,33 +35,32 @@ import cPickle
 import zlib
 from factors import Factor, _find_levels
 import sys
-import ordereddict
-
 
 class DataFrame(object):
     """An implemention of an almost R like DataFrame object.
 
-    Usage:
+    Usage::
+    
         u = DataFrame( { "Field1": [1, 2, 3], 
                         "Field2": ['abc', 'def', 'hgi']},
                         optional:
                          ['Field1', 'Field2']
                          ["rowOne", "rowTwo", "thirdRow"])
 
-        A DataFrame is basically a table with rows and columns.
-        Columns are named, rows are numbered (but can be named)
-        and can be easily selected and calculated upon.
-        Internally, columns are stored as 1d numpy arrays.
-        If you set row names, they're converted into a dictionary
-        for fast access.
-        
-        There is a rich subselection/slicing API,
-        see help(DataFrame.__get_item) (it also works for setting values).
-        Please note that any slice get's you another DataFrame,
-        to access individual entries use get_row(), get_column(), get_value().
-        DataFrames also understand basic arithmetic and you can either
-        add (multiply,...) a constant value, or another DataFrame of the same 
-        size / with the same column names.
+    A DataFrame is basically a table with rows and columns.
+    Columns are named, rows are numbered (but can be named)
+    and can be easily selected and calculated upon.
+    Internally, columns are stored as 1d numpy arrays.
+    If you set row names, they're converted into a dictionary
+    for fast access.
+    
+    There is a rich subselection/slicing API,
+    see help(DataFrame.__get_item) (it also works for setting values).
+    Please note that any slice get's you another DataFrame,
+    to access individual entries use get_row(), get_column(), get_value().
+    DataFrames also understand basic arithmetic and you can either
+    add (multiply,...) a constant value, or another DataFrame of the same 
+    size / with the same column names.
 
     """
     def __init__(self, value_dict=None, columns_ordered=None, row_names_ordered = None ):
@@ -351,7 +350,10 @@ class DataFrame(object):
 
         """
         column_name = self._map_column_number_to_name(column_name)
-        return self.value_dict[column_name]
+        try:
+            return self.value_dict[column_name]
+        except KeyError:
+            raise KeyError("Column not found: %s, available: %s" % (column_name, self.columns_ordered))
 
     def get_value(self, row_idx, column_name):
         """Return the value of a cell.
@@ -383,7 +385,8 @@ class DataFrame(object):
         """Return the data storage as [ [..., ],... ].
 
         This is useful if you want to set from a DataFrame ignoring
-        the column names:
+        the column names::
+        
             df_one = DataFrame({"A": (1, 2),'B': (3, 4)})
             df_two = DataFrame({"b": (1, 2), 'D': (3, 4)})
             df_one[:,:] =  df_two -> ValueError
@@ -507,21 +510,22 @@ class DataFrame(object):
     def __getitem__(self, selection):
         """DateFrames handle an R inspired subset selection.
 
-        Examples: 
-        df[0] - 1xX DateFrame 
-        df[0:2] - 2xX DateFrame 
-        df[0:8:2] - 2xX DateFrame, every second row.
-        df[0, 'Field1'] = 1x1 DataFrame
-        df[:, "Field2"] = Xby1 DataFrame
-        df[:, ("Field2", "Field1")] = DataFrame containing just 
-                                      Field2, Field1 in the given order
-        df[:, 0:1] - Xx2 DataFrame containing columns no. 0 and 1
-        df[(0, 3, 5), (0, 2)] = 3x2 DataFrame with rows 0, 3 and 5 and 
-                                the fields in order 0 and 2
-        df[(True, True, False)] = 2xX DataFrame with just rows 0 and 1
-        df[:, [x.startswith('shu') for x in df.columns_ordered]) = 
-            XxY DataFrame containing just the columns which's fieldnames 
-            start with 'shu'
+        Examples::
+         
+            df[0] - 1xX DateFrame 
+            df[0:2] - 2xX DateFrame 
+            df[0:8:2] - 2xX DateFrame, every second row.
+            df[0, 'Field1'] = 1x1 DataFrame
+            df[:, "Field2"] = Xby1 DataFrame
+            df[:, ("Field2", "Field1")] = DataFrame containing just 
+                                          Field2, Field1 in the given order
+            df[:, 0:1] - Xx2 DataFrame containing columns no. 0 and 1
+            df[(0, 3, 5), (0, 2)] = 3x2 DataFrame with rows 0, 3 and 5 and 
+                                    the fields in order 0 and 2
+            df[(True, True, False)] = 2xX DataFrame with just rows 0 and 1
+            df[:, [x.startswith('shu') for x in df.columns_ordered]) = 
+                XxY DataFrame containing just the columns which's fieldnames 
+                start with 'shu'
 
         """
         columns_to_select, rows_to_select = self._interpret_selection(selection)
@@ -539,18 +543,26 @@ class DataFrame(object):
     def __setitem__(self, selection, value):
         """Set values in a DataFrame.
 
-        You can set single rows:
+        You can set single rows::
+        
             df[3, "Field2"] = 55
-        or columns with a sequence
+            
+        or columns with a sequence::
+        
             df[:, "Field2"] = [1, 2, 3]
-        or arbitary subsections with other DataFrames
+            
+        or arbitary subsections with other DataFrames::
+        
             df[(1, 2), ("Field1", Field2")] = 
                 DataFrame({"Field2": ..., "Field1": ...})
-            (Note: Need to have identical column names, except if they're both single columned!)
-        or from a nested sequence
+            
+        (Note: Need to have identical column names, except if they're both single columned!)
+        or from a nested sequence::
+        
             df[(1, 2), ("Field1", Field2")] = 
                 [(3, 4), (5, 6)]
-            (also see DataFrame.get_as_list_of_lists())
+                
+        (also see :py:func:`get_as_list_of_lists`)
 
         
         """
@@ -660,11 +672,13 @@ class DataFrame(object):
         """Iterate a single column DataFrame.
 
         Raises ValueError on more than one column.
-        Please use these instead:
+        Please use these instead::
+        
             df.iter_values_columns_first()
             df.iter_values_rows_first(),
             df.iter_rows()
-            df.iter_columns()"""
+            df.iter_columns()
+        """
         try:
             self._assert_single_columned()
             return self.iter_values_columns_first()
@@ -989,12 +1003,22 @@ class DataFrame(object):
         """Return the column names (in order)."""
         return self.columns_ordered[:] # return copy
 
-    def as2DMatrix(self, dtype=None):
+    def as_2d_matrix(self, dtype=None):
         """Return all columns as 2d(nRows, nCols)-numpy matrix.
 
         Default dtype is float64
         Raises a ValueError if not all columns could be converted.
         """
+        return self.as2DMatrix()
+
+    def as2DMatrix(self, dtype=None):
+        """Return all columns as 2d(nRows, nCols)-numpy matrix.
+        Please use pep 8 conform as_2d_matrix()
+
+        Default dtype is float64
+        Raises a ValueError if not all columns could be converted.
+        """
+    
         if dtype is None:
             dtype = numpy.float
         res = numpy.zeros(self.dim(),dtype=numpy.float64)
@@ -1084,9 +1108,14 @@ class DataFrame(object):
                     #string columns that have few very large strings
                     #but a large collection of shorter strings
 
-                    if self.value_dict[key].dtype.str.startswith('|O') or self.value_dict[key].dtype.str.startswith("|S"):
+                    if ((self.value_dict[key].dtype.str.startswith('|O')
+                        and (len(self) > 0 and isinstance(self.value_dict[key][0],str) ))
+                            or self.value_dict[key].dtype.str.startswith("|S")):
                         op_data.write('s')
                         op_data.write(zlib.compress( cPickle.dumps(tuple([str(x) for x in self.value_dict[key]]), cPickle.HIGHEST_PROTOCOL)))
+                    elif self.value_dict[key].dtype.str.startswith('|O'): #not as string, but an object...
+                            op_data.write('o')
+                            numpy.save(op_data, numpy.ma.getdata(self.value_dict[key]))
                     else:
                         op_data.write('n')
                         numpy.save(op_data, numpy.ma.getdata(self.value_dict[key]))
@@ -1121,6 +1150,8 @@ class DataFrame(object):
                         except:
                             import pdb
                             pdb.post_mortem()
+                    elif data_type == 'o':
+                        data = numpy.load(op_data)
                     elif data_type == 'n':
                         data = numpy.load(op_data)
                     else: #we're abusing that pickled numpy data starts with \x93NUMPY to preserve backwards compability
@@ -1164,6 +1195,8 @@ class DataFrame(object):
         result = { measurement_name_target: [], value_target: []}
         for id_var in id_vars:
             result[id_var] = []
+        if measure_vars is None:
+            measure_vars = [x for x in self.columns_ordered if not x in id_vars]
         for row in self.iter_rows():
             for measure_var in measure_vars:
                 for id_var in id_vars:
@@ -1310,14 +1343,20 @@ def _convert_to_numpy(seq, column_name):
     #any large uint -> large uint
     #any bool -> bolo
     #other -> object
+    zero_value = 0
     if (unicode in types_present):
-        seq = [s is None and "" or unicode(s) for s in seq]
+        seq = [None if (s is None or (isinstance(s, numpy.ma.MaskedArray) and s.mask)) else 
+                        (s if isinstance(s, unicode) else unicode(s)) for s in seq]
         max_length = max(len(unicode(s)) for s in seq if not s is None)
         dtype = numpy.object
+        zero_value = ''
     elif (str in types_present or numpy.string_ in types_present):
         max_length = max(len(str(s)) for s in seq if not s is None)
-        seq = [s is None and "" or str(s) for s in seq]
+        seq = [None if (s is None or (isinstance(s, numpy.ma.MaskedArray) and s.mask)) else 
+                        (s if isinstance(s, str) else str(s)) for s in seq]
+        #seq = [s is None and "" or str(s) for s in seq]
         dtype = numpy.object
+        zero_value = ''
     elif (numpy.float64 in types_present):
         dtype= numpy.float64
     elif float in types_present or numpy.float in types_present:
@@ -1354,7 +1393,7 @@ def _convert_to_numpy(seq, column_name):
         res = numpy.ma.zeros(len(seq), dtype=numpy.dtype(dtype))
     except TypeError,e:
         raise TypeError(e.message + ' column name %s, dtype: %s' % ( column_name, dtype))
-    res[:] = [(not x is None) and x or 0 for x in seq]
+    res[:] = [(not x is None) and x or zero_value for x in seq]
     res.mask = [(x is None or (hasattr(x,'mask') and x.mask)) for x in seq]
     return res
 
