@@ -736,6 +736,18 @@ class DataFrameCSVTest(unittest.TestCase):
         o.close()
         os.unlink('test.txt')
         self.assertEqual(a, b)
+    
+    def testReadWrite_handling_quotes_single_column(self):
+        conv = DF2CSV()
+        a = DataFrame( { "Field2": ['def','ab','jl','ab','h gi','pqr']})
+        o = open("test.txt", "wb")
+        conv.write(a,o)
+        o.close()
+        o = open("test.txt", "rb")
+        b = conv.read(o, handle_quotes=True)
+        o.close()
+        #os.unlink('test.txt')
+        self.assertEqual(a, b)
 
     def testReadWriteFilename(self):
         conv = DF2CSV()
